#version 330 core

#define NR_INT 2

#define MAX_NR_POINT_LIGHTS 10
#define MAX_NR_SPOT_LIGHTS 10

struct Material
{
	sampler2D texture_diffuse1;
	sampler2D texture_specular1;
    float shininess;
}; 

struct DirLight
{
    vec3 direction;
	
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

struct PointLight
{
    vec3 position;
    
    float constant;
    float linear;
    float quadratic;
	
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

struct SpotLight
{
    vec3 position;
    vec3 direction;
    float cutOff;
    float outerCutOff;
  
    float constant;
    float linear;
    float quadratic;
  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;       
};

in VS_OUT
{
    vec3 Normal;
	vec3 FragPos;
	vec2 TexCoords;
	vec4 FragPosLightSpace;
} fs_in;

uniform vec3 viewPos;
uniform DirLight dirLight;
uniform PointLight pointLights[MAX_NR_POINT_LIGHTS];
uniform SpotLight spotLights[MAX_NR_SPOT_LIGHTS];
uniform Material material;
uniform int number_of_pointlights;
uniform int number_of_spotlights;

// Function prototypes
vec3 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir);
vec3 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir);
vec3 CalcSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 viewDir);

void main()
{    
	int temp;
	
    // Properties
    vec3 norm = normalize(fs_in.Normal);
    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
    
    // Directional lighting
    vec3 result = CalcDirLight(dirLight, norm, viewDir);
    // Point lights
	temp = number_of_pointlights;
	if (temp > MAX_NR_POINT_LIGHTS)
		temp = MAX_NR_POINT_LIGHTS;
    for(int i = 0; i < temp; i++)
        result += CalcPointLight(pointLights[i], norm, fs_in.FragPos, viewDir);    
    // Spot light
	temp = number_of_spotlights;
	if (temp > MAX_NR_SPOT_LIGHTS)
		temp = MAX_NR_SPOT_LIGHTS;
	for(int i = 0; i < temp; i++)
		result += CalcSpotLight(spotLights[i], norm, fs_in.FragPos, viewDir);    

	if(gl_FrontFacing)
	{
		float scale = max(result.r,max(result.g, result.b));

		if (result.r > 0.95)
			result.r = 1;
		else if (result.r > 0.45)
			result.r = 0.7;
		else
			result.r = 0.3;

		if (result.g > 0.95)
			result.g = 1;
		else if (result.g > 0.45)
			result.g = 0.7;
		else
			result.g = 0.3;

		if (result.b > 0.95)
			result.b = 1;
		else if (result.b > 0.45)
			result.b = 0.7;
		else
			result.b = 0.3;

		//ceil(result * NR_INT)/ NR_INT
        //gl_FragColor = vec4(vec3(scale), 1.0)*texture(material.texture_diffuse1, fs_in.TexCoords);
		gl_FragColor = vec4(result.r,result.g,result.b, 1.0)*texture(material.texture_diffuse1, fs_in.TexCoords);
		//gl_FragColor = vec4(vec3(shadow), 1.0f);
		//intensity = vec4(vec3(scale),1.0);
		//gl_FragColor = 2*scale*vec4(result, 1.0)*texture(material.texture_diffuse1, fs_in.TexCoords);
		//gl_FragColor = vec4(norm, 1.0);
	}
    else
	{
        gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
		//intensity = vec4(0.0, 0.0, 0.0, 1.0);
	}	
}

// Calculates the color when using a directional light.
vec3 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // Diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // Specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // Combine results
    vec3 ambient = light.ambient;// * vec3(texture(material.texture_diffuse1, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff;// * vec3(texture(material.texture_diffuse1, fs_in.TexCoords));
    //vec3 diffuse = light.diffuse;
	vec3 specular = light.specular * spec * vec3(texture(material.texture_specular1, fs_in.TexCoords));
	//vec3 specular = light.specular * vec3(texture(material.texture_specular1, fs_in.TexCoords));
    return (ambient + diffuse + specular);
}

// Calculates the color when using a point light.
vec3 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(light.position - fragPos);
    // Diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // Specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // Attenuation
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));    
    // Combine results
    vec3 ambient = light.ambient;// * vec3(texture(material.texture_diffuse1, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff;// * vec3(texture(material.texture_diffuse1, fs_in.TexCoords));
    vec3 specular = light.specular * spec * vec3(texture(material.texture_specular1, fs_in.TexCoords));
    ambient *= attenuation;
    diffuse *= attenuation;
    specular *= attenuation;
    return (ambient + diffuse + specular);
}

// Calculates the color when using a spot light.
vec3 CalcSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(light.position - fragPos);
    // Diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // Specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // Attenuation
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));    
    // Spotlight intensity
    float theta = dot(lightDir, normalize(-light.direction)); 
    float epsilon = light.cutOff - light.outerCutOff;
    float intensity = clamp((theta - light.outerCutOff) / epsilon, 0.0, 1.0);
    // Combine results
    vec3 ambient = light.ambient;// * vec3(texture(material.texture_diffuse1, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff;// * vec3(texture(material.texture_diffuse1, fs_in.TexCoords));
    vec3 specular = light.specular * spec * vec3(texture(material.texture_specular1, fs_in.TexCoords));
    ambient *= attenuation * intensity;
    diffuse *= attenuation * intensity;
    specular *= attenuation * intensity;
    return (ambient + diffuse + specular);
}